/**
 * {解析当前系统环境}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/07/18 16:16:02
 */

const UNKNOW = 'Unknow'

export const UA = navigator.userAgent.toLowerCase()
export const UNKNOW_MATCH = [null, UNKNOW]

/* 分辨率 */
const SCREEN = `${screen.width} x ${screen.height}`

/* 操作系统 */
export const IS_LINUX = UA.includes('linux')
export const IS_WIN = UA.includes('windows nt')
export const IS_MACOS = UA.includes('mac os x')

export const IS_ANDROID = UA.includes('android')
export const IS_IOS =
  UA.includes('iphone') || UA.includes('ipad') || UA.includes('ipod')
export const IS_MOBILE = IS_ANDROID || IS_IOS || UA.includes('mobile')

const OS_NAME = IS_ANDROID
  ? 'Android'
  : IS_IOS
  ? 'IOS'
  : IS_LINUX
  ? 'Linux'
  : IS_WIN
  ? 'Windows'
  : IS_MACOS
  ? 'MacOS'
  : UNKNOW
const OS_VERSION = IS_ANDROID ? UA.match(/android ([\d\.]*?);/)[1] : ''

/* 浏览器 */
export const IS_CHROME = !!window.chrome
export const IS_FIREFOX = 'MozAppearance' in document.documentElement.style
export const IS_SAFARI = (IS_MACOS || IS_IOS) && !!window.safari

/* 浏览器名称 */
const BROWSER_NAME = IS_SAFARI
  ? 'Safari'
  : IS_CHROME
  ? 'Chrome'
  : IS_FIREFOX
  ? 'Firefox'
  : 'Unknow'

/* 浏览器版本 */
const GENERAL_VERSION = UA.match(/version\/([\d\.]*?) /) || UNKNOW_MATCH
const CHROME_VERSION = UA.match(/chrome\/([\d\.]*?) /) || UNKNOW_MATCH
const IPAD_WEBKIT_VERSION = UA.match(/crios\/([\d\.]*?) /) || GENERAL_VERSION
const FIREFOX_VERSION = UA.match(/firefox\/([\d\.]*)/) || UNKNOW_MATCH

const BROWSER_VERSION = IS_IOS
  ? IPAD_WEBKIT_VERSION[1]
  : IS_CHROME
  ? CHROME_VERSION[1]
  : IS_FIREFOX
  ? FIREFOX_VERSION[1]
  : GENERAL_VERSION[1]

// 是否支持 Import assertions
export const SUPPORT_ASSERTIONS = IS_CHROME
  ? CHROME_VERSION[1].split('.').shift() > 91
  : IS_SAFARI
  ? GENERAL_VERSION[1].split('.').shift() >= 15
  : false

export { OS_NAME, OS_VERSION, BROWSER_NAME, BROWSER_VERSION, SCREEN }
