import { reactive } from 'vue'

import * as env from '@/lib/env.js'
import { checkCanvas, checkMedia } from '@/lib/checks.js'

const store = reactive({
  scores: 80,
  useragent: navigator.userAgent,
  env,

  modules: [
    {
      name: '音频支持',
      total: 0,
      factor: 70,
      scores: [
        !!window.AudioContext ^ 0,
        checkMedia('audio/wav'),
        checkMedia('audio/mpeg'),
        checkMedia('audio/flac'),
        checkMedia('audio/aac'),
        checkMedia('audio/ogg'),
        checkMedia('audio/webm')
      ],
      items: [
        'Web Audio API',
        'pcm audio support',
        'mp3 support',
        'flac support',
        'aac support',
        'ogg support',
        'webm support'
      ]
    },
    {
      name: '视频支持',
      total: 0,
      factor: 70,
      scores: [
        checkMedia('video/mp4; codecs="mp4v.20.8"', 0),
        checkMedia('video/mp4', 0),
        checkMedia('video/mp4; codecs="hev1"', 0) * 1.1,
        checkMedia('video/ogg', 0),
        checkMedia('video/webm; codecs="vp8"', 0),
        checkMedia('video/webm; codecs="vp9"', 0)
      ],
      items: [
        'MPEG-4 ASP support',
        'H.264 support',
        'H.265 support',
        'Ogg Theora support',
        'WebM with VP8 support',
        'WebM with VP9 support'
      ]
    },
    {
      name: '流媒体',
      total: 0,
      scores: [
        1,
        !!navigator.requestMediaKeySystemAccess ^ 0,
        MediaSource.isTypeSupported(
          'video/mp4; codecs="avc1.42E01E, mp4a.40.2"'
        )
          ? 1.1
          : 0,
        MediaSource.isTypeSupported(
          'video/mp4; codecs="avc1.42E01E, mp4a.40.2"'
        )
          ? 1.1
          : 0
      ],
      factor: 100,
      items: [
        'Media Source extensions',
        'DRM support',
        'Dynamic Adaptive Streaming / MPEG-DASH',
        'HTTP Live Streaming / HLS'
      ]
    },
    {
      name: '网络',
      total: 0,
      factor: 60,
      scores: [
        !!window.EventSource ^ 0,
        !!window.WebSocket ^ 0,
        !!window.fetch ^ 0,
        !!File.prototype.arrayBuffer ^ 0
      ],
      items: [
        'Server-Sent events',
        'websocket',
        'fetch api',
        'ArrayBuffer and Blob support'
      ]
    },
    {
      name: '数据流',
      factor: 80,
      total: 0,
      scores: [
        !!window.FileSystemHandle ^ 0,
        !!window.ReadableStream ^ 0,
        !!window.WritableStream ^ 0
      ],
      items: ['File System', 'Readable Stream', 'Writable Stream']
    },
    {
      name: '自定义组件',
      total: 0,
      factor: 80,
      scores: [
        !!window.customElements ^ 0,
        !!document.body.attachShadow ^ 0,
        document.adoptedStyleSheets ? 1.1 : 0
      ],
      items: ['Custom elements', 'Shadow DOM', 'adoptedStyleSheets']
    },
    {
      name: '图形图象支持',
      total: 0,
      factor: 120,
      scores: [
        checkCanvas('2d'),
        checkCanvas('webgl'),
        checkCanvas('webgl2') * 1.1,
        checkCanvas('webgpu') * 2,
        navigator.xr ? 2 : 0
      ],
      items: ['canvas 2D', 'WebGL', 'WebGL 2', 'WebGPU', 'WebVR']
    },
    {
      name: '点对点连接',
      total: 0,
      factor: 40,
      scores: [
        !!(
          window.RTCPeerConnection ||
          window.webkitRTCPeerConnection ||
          window.mozRTCPeerConnection
        ) ^ 0,
        window.RTCIceTransport ||
        window.webkitRTCIceTransport ||
        window.mozRTCIceTransport
          ? 2.5
          : 0,
        !!navigator.getUserMedia ^ 0,
        navigator.mediaDevices && navigator.mediaDevices.getDisplayMedia
          ? 1.5
          : 0,
        !!window.MediaRecorder ^ 0
      ],
      items: [
        'WebRTC 1.0',
        'ObjectRTC API for WebRTC',
        'Access the webcam',
        'Screen Capture',
        'Media Stream recorder'
      ]
    },
    {
      name: 'ES6模块',
      total: 0,
      factor: 40,
      scores: [1, env.SUPPORT_ASSERTIONS ? 2.5 : 0],
      items: ['import/export', 'Import assertions']
    },
    {
      name: '性能',
      total: 0,
      factor: 70,
      scores: [!!navigator.serviceWorker ^ 0, window.WebAssembly ? 1.5 : 0],
      items: ['Web Workers', 'WebAssembly']
    },
    {
      name: '其他',
      total: 0,
      factor: 80,
      scores: [
        !!navigator.bluetooth ^ 0,
        navigator.clipboard ? 0.8 : 0,
        navigator.usb ? 2 : 0,
        !!navigator.windowControlsOverlay ^ 0
      ],
      items: ['Bluetooth', 'clipboard', 'USB', 'Window Controls Overlay']
    }
  ]
})

store.modules.forEach(
  it => (
    (it.total = it.items.length),
    (it.passed = it.scores.filter(n => n > 0).length)
  )
)
store.scores = store.modules.map(it => it.scores.sum() * it.factor).sum()

export default function (app) {
  app.config.globalProperties.$store = store
}
